;//-------------------------------------------------------------------
;//   Microsoft  Windows
;//   Copyright  (C) Microsoft Corporation, 2004.
;//   File....:  bcrypt.h
;//   Contents:  Cryptographic Primitive API Prototypes and Definitions
;//--------------------------------------------------------------------

 BCRYPT_SUCCESS               = 0
 BCRYPT_OBJECT_ALIGNMENT      = 16
 BCRYPT_OPERATION_TYPE_HASH   = 1         ;//    - BCRYPT_MULTI_HASH_OPERATION


 BCRYPT_KDF_HASH              du  'HASH',0
 BCRYPT_KDF_HMAC              du  'HMAC',0
 BCRYPT_KDF_RAW_SECRET        du  'TRUNCATE',0
 BCRYPT_KDF_SP80056A_CONCAT   du  'SP800_56A_CONCAT',0
 BCRYPT_KDF_TLS_PRF           du  'TLS_PRF',0

; //
; //   KDF BufferTypes
; //   BCRYPT_KDF_HASH  BCRYPT_KDF_HMAC 
;//       -   KDF_SECRET_PREPEND  KDF_SECRET_APPEND. 
;// BufferTypes       .
;// 
 KDF_HASH_ALGORITHM       = 0x0
 KDF_SECRET_PREPEND       = 0x1
 KDF_SECRET_APPEND        = 0x2
 KDF_HMAC_KEY             = 0x3
 KDF_TLS_PRF_LABEL        = 0x4
 KDF_TLS_PRF_SEED         = 0x5
 KDF_SECRET_HANDLE        = 0x6
 KDF_TLS_PRF_PROTOCOL     = 0x7
 KDF_ALGORITHMID          = 0x8
 KDF_PARTYUINFO           = 0x9
 KDF_PARTYVINFO           = 0xA
 KDF_SUPPPUBINFO          = 0xB
 KDF_SUPPPRIVINFO         = 0xC
 KDF_LABEL                = 0xD
 KDF_CONTEXT              = 0xE
 KDF_SALT                 = 0xF
 KDF_ITERATION_COUNT      = 0x10
 KDF_GENERIC_PARAMETER    = 0x11
 KDF_KEYBITLENGTH         = 0x12

 KDF_USE_SECRET_AS_HMAC_KEY_FLAG  = 0x1

;// 
;//  BCrypt String Properties
;//  BCrypt(Import/Export)Key BLOB types

 BCRYPT_OPAQUE_KEY_BLOB      du  'OpaqueKeyBlob'
 BCRYPT_KEY_DATA_BLOB        du  'KeyDataBlob'
 BCRYPT_AES_WRAP_KEY_BLOB    du  'Rfc3565KeyWrapBlob'

 BCRYPT_AUTH_CIPHER_MODE_INFO_VERSION  = 0x00000001
 BCRYPT_AUTH_MODE_CHAIN_CALLS_FLAG     = 0x00000001
 BCRYPT_AUTH_MODE_IN_PROGRESS_FLAG     = 0x00000002

;// 
;//  Common algorithm Unicode identifiers
;// 
 BCRYPT_3DES_112_ALGORITHM               du  '3DES_112',0
 BCRYPT_3DES_ALGORITHM                   du  '3DES',0
 BCRYPT_AES_ALGORITHM                    du  'AES',0
 BCRYPT_AES_CMAC_ALGORITHM               du  'AES-CMAC',0
 BCRYPT_AES_GMAC_ALGORITHM               du  'AES-GMAC',0
 BCRYPT_AES_XTS_ALGORITHM                du  'XTS-AES',0
 BCRYPT_CAPI_KDF_ALGORITHM               du  'CAPI_KDF',0
 BCRYPT_DES_ALGORITHM                    du  'DES',0
 BCRYPT_DESX_ALGORITHM                   du  'DESX',0
 BCRYPT_DH_ALGORITHM                     du  'DH',0
 BCRYPT_DSA_ALGORITHM                    du  'DSA',0
 BCRYPT_ECDH_ALGORITHM                   du  'ECDH',0
 BCRYPT_ECDH_P256_ALGORITHM              du  'ECDH_P256',0
 BCRYPT_ECDH_P384_ALGORITHM              du  'ECDH_P384',0
 BCRYPT_ECDH_P521_ALGORITHM              du  'ECDH_P521',0
 BCRYPT_ECDSA_ALGORITHM                  du  'ECDSA',0
 BCRYPT_ECDSA_P256_ALGORITHM             du  'ECDSA_P256',0
 BCRYPT_ECDSA_P384_ALGORITHM             du  'ECDSA_P384',0
 BCRYPT_ECDSA_P521_ALGORITHM             du  'ECDSA_P521',0
 BCRYPT_MD2_ALGORITHM                    du  'MD2',0
 BCRYPT_MD4_ALGORITHM                    du  'MD4',0
 BCRYPT_MD5_ALGORITHM                    du  'MD5',0
 BCRYPT_PBKDF2_ALGORITHM                 du  'PBKDF2',0
 BCRYPT_RC2_ALGORITHM                    du  'RC2',0
 BCRYPT_RC4_ALGORITHM                    du  'RC4',0
 BCRYPT_RNG_ALGORITHM                    du  'RNG',0
 BCRYPT_RNG_DUAL_EC_ALGORITHM            du  'DUALECRNG',0
 BCRYPT_RNG_FIPS186_DSA_ALGORITHM        du  'FIPS186DSARNG',0
 BCRYPT_RSA_ALGORITHM                    du  'RSA',0
 BCRYPT_RSA_SIGN_ALGORITHM               du  'RSA_SIGN',0
 BCRYPT_SHA1_ALGORITHM                   du  'SHA1',0
 BCRYPT_SHA256_ALGORITHM                 du  'SHA256',0
 BCRYPT_SHA384_ALGORITHM                 du  'SHA384',0
 BCRYPT_SHA512_ALGORITHM                 du  'SHA512',0
 BCRYPT_SP800108_CTR_HMAC_ALGORITHM      du  'SP800_108_CTR_HMAC',0
 BCRYPT_SP80056A_CONCAT_ALGORITHM        du  'SP800_56A_CONCAT',0
 BCRYPT_TLS1_1_KDF_ALGORITHM             du  'TLS1_1_KDF',0
 BCRYPT_TLS1_2_KDF_ALGORITHM             du  'TLS1_2_KDF',0

;// AES Property Strings (  30 )

 BCRYPT_CHAIN_MODE_CCM       du  'ChainingModeCCM',0   ;// Auth = 16 byte, IV = 12 byte
 BCRYPT_CHAIN_MODE_GCM       du  'ChainingModeGCM',0   ;// Auth = 16 byte, IV = 12 byte

 BCRYPT_CHAIN_MODE_CBC       du  'ChainingModeCBC',0   ;// not Auth, IV = 16 byte
 BCRYPT_CHAIN_MODE_CFB       du  'ChainingModeCFB',0   ;// not Auth, IV = 16 byte

 BCRYPT_CHAIN_MODE_ECB       du  'ChainingModeECB',0   ;// not Auth, not IV
 BCRYPT_CHAIN_MODE_NA        du  'ChainingModeN/A',0

;// BCryptGetProperty strings (Set   CHAINING_MODE)
;// https://docs.microsoft.com/en-us/windows/win32/seccng/cng-property-identifiers 

 BCRYPT_ALGORITHM_NAME                du  'AlgorithmName',0         ;//  - 
 BCRYPT_AUTH_TAG_LENGTH               du  'AuthTagLength',0         ;// BCRYPT_AUTH_TAG_LENGTHS_STRUCT ( )
 BCRYPT_BLOCK_LENGTH                  du  'BlockLength',0           ;// DWORD -    
 BCRYPT_BLOCK_SIZE_LIST               du  'BlockSizeList',0         ;//  DWORD,   = /4
 BCRYPT_CHAINING_MODE                 du  'ChainingMode',0          ;//   AES
 BCRYPT_DH_PARAMETERS                 du  'DHParameters',0          ;// BCRYPT_DH_PARAMETER_HEADER struct ( -)
 BCRYPT_DSA_PARAMETERS                du  'DSAParameters',0         ;// BCRYPT_DSA_PARAMETER_HEADER struct
 BCRYPT_EFFECTIVE_KEY_LENGTH          du  'EffectiveKeyLength',0    ;// DWORD -   RC2
 BCRYPT_HASH_BLOCK_LENGTH             du  'HashBlockLength',0       ;// DWORD -    -
 BCRYPT_HASH_LENGTH                   du  'HashDigestLength',0      ;// DWORD -    - 
 BCRYPT_HASH_OID_LIST                 du  'HashOIDList',0           ;// BCRYPT_OID_LIST struct
 BCRYPT_INITIALIZATION_VECTOR         du  'IV',0                    ;//  IV,    
 BCRYPT_KEY_LENGTH                    du  'KeyLength',0             ;// DWORD -     
 BCRYPT_KEY_LENGTHS                   du  'KeyLengths',0            ;// BCRYPT_KEY_LENGTHS_STRUCT -   
 BCRYPT_KEY_STRENGTH                  du  'KeyStrength',0           ;// DWORD -     
 BCRYPT_MESSAGE_BLOCK_LENGTH          du  'MessageBlockLength',0    ;//    CFB (=1  8- CFB)
 BCRYPT_MULTI_OBJECT_LENGTH           du  'MultiObjectLength',0     ;// BCRYPT_MULTI_OBJECT_LENGTH_STRUCT (  )
 BCRYPT_OBJECT_LENGTH                 du  'ObjectLength',0          ;// DWORD -     
 BCRYPT_PADDING_SCHEMES               du  'PaddingSchemes',0        ;//    RSA
 BCRYPT_PROVIDER_HANDLE               du  'ProviderHandle',0        ;// DWORD -    
 BCRYPT_SIGNATURE_LENGTH              du  'SignatureLength',0       ;// DWORD -      

 BCRYPT_IS_KEYED_HASH                 du  'IsKeyedHash',0           ;// 
 BCRYPT_IS_REUSABLE_HASH              du  'IsReusableHash',0        ;//
 BCRYPT_PCP_PLATFORM_TYPE_PROPERTY    du  'PCP_PLATFORM_TYPE',0     ;//
 BCRYPT_PCP_PROVIDER_VERSION_PROPERTY du  'PCP_PROVIDER_VERSION',0  ;//
 BCRYPT_PRIMITIVE_TYPE                du  'PrimitiveType',0         ;//
 BCRYPT_PUBLIC_KEY_LENGTH             du  'PublicKeyLength',0       ;// 


;// Supported RSA Padding Types

 BCRYPT_SUPPORTED_PAD_ROUTER      = 0x00000001
 BCRYPT_SUPPORTED_PAD_PKCS1_ENC   = 0x00000002
 BCRYPT_SUPPORTED_PAD_PKCS1_SIG   = 0x00000004
 BCRYPT_SUPPORTED_PAD_OAEP        = 0x00000008
 BCRYPT_SUPPORTED_PAD_PSS         = 0x00000010

;// BCrypt Flags

 BCRYPT_PROV_DISPATCH         = 0x00000001  ;//  BCryptOpenAlgorithmProvider
 BCRYPT_BLOCK_PADDING         = 0x00000001  ;//  BCryptEncrypt/Decrypt

;// RSA padding schemes

 BCRYPT_PAD_NONE              = 0x00000001
 BCRYPT_PAD_PKCS1             = 0x00000002  ;//  BCryptEncrypt/Decrypt BCryptSignHash/VerifySignature
 BCRYPT_PAD_OAEP              = 0x00000004  ;//  BCryptEncrypt/Decrypt
 BCRYPT_PAD_PSS               = 0x00000008  ;//  BCryptSignHash/VerifySignature
 BCRYPT_PAD_PKCS1_OPTIONAL_HASH_OID   = 0x00000010   ;// BCryptVerifySignature

 BCRYPTBUFFER_VERSION         = 0

;// 
;//  Structures used to represent key blobs.
;// 
 BCRYPT_PUBLIC_KEY_BLOB       du  'PUBLICBLOB',0
 BCRYPT_PRIVATE_KEY_BLOB      du  'PRIVATEBLOB',0

;//  BLOB- BCRYPT_RSAPUBLIC_BLOB  BCRYPT_RSAPRIVATE_BLOB
;//     RSA    . 
;//           RSA.
;// BCRYPT_RSAPRIVATE_BLOB   :
;//  
;// 
;// Prime1
;// Prime2

 BCRYPT_RSAPRIVATE_BLOB      du  'RSAPRIVATEBLOB',0
 BCRYPT_RSAPUBLIC_BLOB       du  'RSAPUBLICBLOB',0
 LEGACY_RSAPRIVATE_BLOB      du  'CAPIPRIVATEBLOB',0
 LEGACY_RSAPUBLIC_BLOB       du  'CAPIPUBLICBLOB',0

 BCRYPT_RSAPUBLIC_MAGIC       = 0x31415352  ;//  RSA1
 BCRYPT_RSAPRIVATE_MAGIC      = 0x32415352  ;//  RSA2

;//  The BCRYPT_RSAFULLPRIVATE_BLOB blob type is used to transport plaintext private RSA keys.

 BCRYPT_RSAFULLPRIVATE_BLOB      du  'RSAFULLPRIVATEBLOB',0
 BCRYPT_RSAFULLPRIVATE_MAGIC     =   0x33415352  ;//  RSA3

;// Properties of secret agreement algorithms

 BCRYPT_GLOBAL_PARAMETERS    du  'SecretAgreementParam',0
 BCRYPT_PRIVATE_KEY          du  'PrivKeyVadu',0

;//  BLOB- BCRYPT_ECCPUBLIC_BLOB  BCRYPT_ECCPRIVATE_BLOB
;//     ECC  . 
;//       ECC.

 BCRYPT_ECCPUBLIC_BLOB           du  'ECCPUBLICBLOB',0
 BCRYPT_ECCPRIVATE_BLOB          du  'ECCPRIVATEBLOB',0
 BCRYPT_ECCFULLPUBLIC_BLOB       du  'ECCFULLPUBLICBLOB',0
 BCRYPT_ECCFULLPRIVATE_BLOB      du  'ECCFULLPRIVATEBLOB',0
 SSL_ECCPUBLIC_BLOB              du  'SSLECCPUBLICBLOB',0

 BCRYPT_ECDH_PUBLIC_P256_MAGIC    = 0x314B4345  ;//  ECK1
 BCRYPT_ECDH_PRIVATE_P256_MAGIC   = 0x324B4345  ;//  ECK2
 BCRYPT_ECDH_PUBLIC_P384_MAGIC    = 0x334B4345  ;//  ECK3
 BCRYPT_ECDH_PRIVATE_P384_MAGIC   = 0x344B4345  ;//  ECK4
 BCRYPT_ECDH_PUBLIC_P521_MAGIC    = 0x354B4345  ;//  ECK5
 BCRYPT_ECDH_PRIVATE_P521_MAGIC   = 0x364B4345  ;//  ECK6
 BCRYPT_ECDH_PUBLIC_GENERIC_MAGIC     = 0x504B4345  ;//  ECKP
 BCRYPT_ECDH_PRIVATE_GENERIC_MAGIC    = 0x564B4345  ;//  ECKV

 BCRYPT_ECDSA_PUBLIC_P256_MAGIC   = 0x31534345  ;//  ECS1
 BCRYPT_ECDSA_PRIVATE_P256_MAGIC  = 0x32534345  ;//  ECS2
 BCRYPT_ECDSA_PUBLIC_P384_MAGIC   = 0x33534345  ;//  ECS3
 BCRYPT_ECDSA_PRIVATE_P384_MAGIC  = 0x34534345  ;//  ECS4
 BCRYPT_ECDSA_PUBLIC_P521_MAGIC   = 0x35534345  ;//  ECS5
 BCRYPT_ECDSA_PRIVATE_P521_MAGIC  = 0x36534345  ;//  ECS6
 BCRYPT_ECDSA_PUBLIC_GENERIC_MAGIC    = 0x50444345  ;//  ECDP
 BCRYPT_ECDSA_PRIVATE_GENERIC_MAGIC   = 0x56444345  ;//  ECDV

;// ECC Full versions

 BCRYPT_ECC_FULLKEY_BLOB_V1                  = 0x1
 BCRYPT_NO_CURVE_GENERATION_ALG_ID           = 0x0
 BCRYPT_ECC_PRIME_SHORT_WEIERSTRASS_CURVE    = 0x1
 BCRYPT_ECC_PRIME_TWISTED_EDWARDS_CURVE      = 0x2
 BCRYPT_ECC_PRIME_MONTGOMERY_CURVE           = 0x3


;//  BLOB- BCRYPT_DH_PUBLIC_BLOB  BCRYPT_DH_PRIVATE_BLOB
;//     DH    .   
;//    DH.

 BCRYPT_DH_PRIVATE_BLOB          du  'DHPRIVATEBLOB',0
 BCRYPT_DH_PUBLIC_BLOB           du  'DHPUBLICBLOB',0
 LEGACY_DH_PRIVATE_BLOB          du  'CAPIDHPRIVATEBLOB',0
 LEGACY_DH_PUBLIC_BLOB           du  'CAPIDHPUBLICBLOB',0

 BCRYPT_DH_PUBLIC_MAGIC           = 0x42504844  ;//  DHPB
 BCRYPT_DH_PRIVATE_MAGIC          = 0x56504844  ;//  DHPV

;//  Property Strings for DH

 BCRYPT_DH_PARAMETERS_MAGIC      =   0x4d504844  ;//  DHPM

;//  BLOB- BCRYPT_DSA_PUBLIC_BLOB  BCRYPT_DSA_PRIVATE_BLOB
;//     DSA   . 

 BCRYPT_DSA_PRIVATE_BLOB         du  'DSAPRIVATEBLOB',0
 BCRYPT_DSA_PUBLIC_BLOB          du  'DSAPUBLICBLOB',0
 LEGACY_DSA_PRIVATE_BLOB         du  'CAPIDSAPRIVATEBLOB',0
 LEGACY_DSA_PUBLIC_BLOB          du  'CAPIDSAPUBLICBLOB',0
 LEGACY_DSA_V2_PRIVATE_BLOB      du  'V2CAPIDSAPRIVATEBLOB',0
 LEGACY_DSA_V2_PUBLIC_BLOB       du  'V2CAPIDSAPUBLICBLOB',0

 BCRYPT_DSA_PUBLIC_MAGIC         = 0x42505344  ;//  DSPB
 BCRYPT_DSA_PRIVATE_MAGIC        = 0x56505344  ;//  DSPV
 BCRYPT_DSA_PUBLIC_MAGIC_V2      = 0x32425044  ;//  DPB2
 BCRYPT_DSA_PRIVATE_MAGIC_V2     = 0x32565044  ;//  DPV2

 BCRYPT_KEY_DATA_BLOB_MAGIC      = 0x4d42444b ;// Key Data Blob Magic (KDBM)
 BCRYPT_KEY_DATA_BLOB_VERSION1   = 0x1

;//  Property Strings for DSA
 BCRYPT_DSA_PARAMETERS_MAGIC     =   0x4d505344  ;//  DSPM
 BCRYPT_DSA_PARAMETERS_MAGIC_V2  =   0x324d5044  ;//  DPM2


;// Property Strings for ECC
 BCRYPT_ECC_PARAMETERS           du  'ECCParameters'
 BCRYPT_ECC_CURVE_NAME           du  'ECCCurveName'
 BCRYPT_ECC_CURVE_NAME_LIST      du  'ECCCurveNameList'
 BCRYPT_ECC_PARAMETERS_MAGIC     =   0x50434345  ;//  ECCP

;// 
;//   ECC
;// 
 BCRYPT_ECC_CURVE_25519              du  'curve25519',0
 BCRYPT_ECC_CURVE_BRAINPOOLP160R1    du  'brainpoolP160r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP160T1    du  'brainpoolP160t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP192R1    du  'brainpoolP192r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP192T1    du  'brainpoolP192t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP224R1    du  'brainpoolP224r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP224T1    du  'brainpoolP224t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP256R1    du  'brainpoolP256r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP256T1    du  'brainpoolP256t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP320R1    du  'brainpoolP320r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP320T1    du  'brainpoolP320t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP384R1    du  'brainpoolP384r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP384T1    du  'brainpoolP384t1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP512R1    du  'brainpoolP512r1',0
 BCRYPT_ECC_CURVE_BRAINPOOLP512T1    du  'brainpoolP512t1',0
 BCRYPT_ECC_CURVE_EC192WAPI          du  'ec192wapi',0
 BCRYPT_ECC_CURVE_NISTP192           du  'nistP192',0
 BCRYPT_ECC_CURVE_NISTP224           du  'nistP224',0
 BCRYPT_ECC_CURVE_NISTP256           du  'nistP256',0
 BCRYPT_ECC_CURVE_NISTP384           du  'nistP384',0
 BCRYPT_ECC_CURVE_NISTP521           du  'nistP521',0
 BCRYPT_ECC_CURVE_NUMSP256T1         du  'numsP256t1',0
 BCRYPT_ECC_CURVE_NUMSP384T1         du  'numsP384t1',0
 BCRYPT_ECC_CURVE_NUMSP512T1         du  'numsP512t1',0
 BCRYPT_ECC_CURVE_SECP160K1          du  'secP160k1',0
 BCRYPT_ECC_CURVE_SECP160R1          du  'secP160r1',0
 BCRYPT_ECC_CURVE_SECP160R2          du  'secP160r2',0
 BCRYPT_ECC_CURVE_SECP192K1          du  'secP192k1',0
 BCRYPT_ECC_CURVE_SECP192R1          du  'secP192r1',0
 BCRYPT_ECC_CURVE_SECP224K1          du  'secP224k1',0
 BCRYPT_ECC_CURVE_SECP224R1          du  'secP224r1',0
 BCRYPT_ECC_CURVE_SECP256K1          du  'secP256k1',0
 BCRYPT_ECC_CURVE_SECP256R1          du  'secP256r1',0
 BCRYPT_ECC_CURVE_SECP384R1          du  'secP384r1',0
 BCRYPT_ECC_CURVE_SECP521R1          du  'secP521r1',0
 BCRYPT_ECC_CURVE_WTLS12             du  'wtls12',0
 BCRYPT_ECC_CURVE_WTLS7              du  'wtls7',0
 BCRYPT_ECC_CURVE_WTLS9              du  'wtls9',0
 BCRYPT_ECC_CURVE_X962P192V1         du  'x962P192v1',0
 BCRYPT_ECC_CURVE_X962P192V2         du  'x962P192v2',0
 BCRYPT_ECC_CURVE_X962P192V3         du  'x962P192v3',0
 BCRYPT_ECC_CURVE_X962P239V1         du  'x962P239v1',0
 BCRYPT_ECC_CURVE_X962P239V2         du  'x962P239v2',0
 BCRYPT_ECC_CURVE_X962P239V3         du  'x962P239v3',0
 BCRYPT_ECC_CURVE_X962P256V1         du  'x962P256v1',0

;// 
;//  Microsoft built-in providers.
;// 
 MS_PRIMITIVE_PROVIDER                   du  'Microsoft Primitive Provider',0
 MS_PLATFORM_CRYPTO_PROVIDER             du  'Microsoft Platform Crypto Provider',0

;// 
;//  Interfaces
;// 
 BCRYPT_CIPHER_INTERFACE                  = 0x00000001
 BCRYPT_HASH_INTERFACE                    = 0x00000002
 BCRYPT_ASYMMETRIC_ENCRYPTION_INTERFACE   = 0x00000003
 BCRYPT_SECRET_AGREEMENT_INTERFACE        = 0x00000004
 BCRYPT_SIGNATURE_INTERFACE               = 0x00000005
 BCRYPT_RNG_INTERFACE                     = 0x00000006
 BCRYPT_KEY_DERIVATION_INTERFACE          = 0x00000007  ;// Win8

;// 
;//  Primitive algorithm provider functions.
;// 
 BCRYPT_ALG_HANDLE_HMAC_FLAG              = 0x00000008
 BCRYPT_HASH_REUSABLE_FLAG                = 0x00000020
 BCRYPT_CAPI_AES_FLAG                     = 0x00000010
 BCRYPT_MULTI_FLAG                        = 0x00000040

;// 
; //  BUFFERS_LOCKED,   BCryptEncrypt/Decrypt , 
; //  pbInput  pbOutput   CNG    .
; //       ,     .
;// 
 BCRYPT_BUFFERS_LOCKED_FLAG              = 0x00000040
 BCRYPT_EXTENDED_KEYSIZE                 = 0x00000080
 BCRYPT_ENABLE_INCOMPATIBLE_FIPS_CHECKS  = 0x00000100

;//  AlgOperations flags for use with BCryptEnumAlgorithms()

 BCRYPT_CIPHER_OPERATION                  = 0x00000001  ;//   
 BCRYPT_HASH_OPERATION                    = 0x00000002  ;// 
 BCRYPT_ASYMMETRIC_ENCRYPTION_OPERATION   = 0x00000004  ;// 
 BCRYPT_SECRET_AGREEMENT_OPERATION        = 0x00000008  ;//  
 BCRYPT_SIGNATURE_OPERATION               = 0x00000010  ;// 
 BCRYPT_RNG_OPERATION                     = 0x00000020  ;//  
 BCRYPT_KEY_DERIVATION_OPERATION          = 0x00000040  ;//<----------- Win-8

;// 
;//  Primitive provider interfaces.
;// 
if 0
 BCRYPT_CIPHER_INTERFACE_VERSION_1                   BCRYPT_MAKE_INTERFACE_VERSION(1,0)
 BCRYPT_HASH_INTERFACE_VERSION_1                     BCRYPT_MAKE_INTERFACE_VERSION(1,0)
 BCRYPT_HASH_INTERFACE_MAJORVERSION_2                2
 BCRYPT_HASH_INTERFACE_VERSION_2                     BCRYPT_MAKE_INTERFACE_VERSION(BCRYPT_HASH_INTERFACE_MAJORVERSION_2,0)
 BCRYPT_ASYMMETRIC_ENCRYPTION_INTERFACE_VERSION_1    BCRYPT_MAKE_INTERFACE_VERSION(1,0)
 BCRYPT_SECRET_AGREEMENT_INTERFACE_VERSION_1         BCRYPT_MAKE_INTERFACE_VERSION(1,0)
 BCRYPT_SIGNATURE_INTERFACE_VERSION_1                BCRYPT_MAKE_INTERFACE_VERSION(1,0)
 BCRYPT_RNG_INTERFACE_VERSION_1                      BCRYPT_MAKE_INTERFACE_VERSION(1,0)
end if

;// Flags to BCryptGenRandom
;// BCRYPT_RNG_USE_ENTROPY_IN_BUFFER   Win7+
;//
 BCRYPT_RNG_USE_ENTROPY_IN_BUFFER   = 0x00000001
 BCRYPT_USE_SYSTEM_PREFERRED_RNG    = 0x00000002


;//  CryptoConfig Definitions ######################################

;//  Interface registration flags
 CRYPT_MIN_DEPENDENCIES      = 0x00000001
 CRYPT_PROCESS_ISOLATE       = 0x00010000    ;//  User-mode only

;//  Processor modes supported by a provider
;//  (Valid for BCryptQueryProviderRegistration and BCryptResolveProviders):
 CRYPT_UM                    = 0x00000001    ;//  User mode only
 CRYPT_KM                    = 0x00000002    ;//  Kernel mode only
 CRYPT_MM                    = 0x00000003    ;//  Multi-mode: Must support BOTH UM and KM
 CRYPT_ANY                   = 0x00000004    ;//  Wildcard: Either UM, or KM, or both

;//  Write behavior flags
 CRYPT_OVERWRITE             = 0x00000001
 CRYPT_LOCAL                 = 0x00000001     ;//  Configuration tables
 CRYPT_DOMAIN                = 0x00000002
 CRYPT_EXCLUSIVE             = 0x00000001     ;//  Context configuration flags
 CRYPT_OVERRIDE              = 0x00010000     ;//  Enterprise table only
 CRYPT_ALL_FUNCTIONS         = 0x00000001     ;//  Resolution and enumeration flags
 CRYPT_ALL_PROVIDERS         = 0x00000002
 CRYPT_PRIORITY_TOP          = 0x00000000     ;//  Priority list positions
 CRYPT_PRIORITY_BOTTOM       = 0xFFFFFFFF

;//  Default system-wide context
 CRYPT_DEFAULT_CONTEXT       du  'Default',0

;//#########################################################

struct BCRYPT_ALGORITHM_IDENTIFIER
  pszName           dd  0  ;//
  dwClass           dd  0  ;//
  dwFlags           dd  0  ;//
ends

;//        .
struct _BCRYPT_DH_PARAMETER_HEADER
  cbLength          dd  0  ;//     .
  dwMagic           dd  0  ;//    (0x4d504844 = MPHD)
  cbKeyLength       dd  0  ;//  ,     
ends

struct _BCRYPT_OID_LIST
  dwOIDCount        dd  0  ;// - 
  pOIDs             dd  0  ;//   BCRYPT_OID
ends

struct BCRYPT_KEY_LENGTHS_STRUCT
   dwMinLen        dd  0   ;// .   
   dwMaxLen        dd  0   ;// . 
   dwIncrement     dd  0   ;//    
ends

struct BCRYPT_AUTH_TAG_LENGTHS_STRUCT
   dwMinLen        dd  0   ;// .   
   dwMaxLen        dd  0   ;// . 
   dwIncrement     dd  0   ;//    
ends

struct BCRYPT_AUTH_CIPHER_MODE_INFO
   cbSize          dd   64  ;//   
   dwInfoVer       dd   1   ;//   = 1
   pbNonce         dd   0   ;//   ,  Nonce.
   cbNonce         dd   0   ;//  
   pbAuthData      dd   0   ;//   -
   cbAuthData      dd   0   ;// 
   pbTag           dd   0   ;//   -
   cbTag           dd   0   ;//
   pbMacContext    dd   0   ;//     , (   ) 
   cbMacContext    dd   0   ;//
   cbAAD           dd   0   ;// AAD (   )
   cbData          dq   0   ;//     (   )
   dwFlags         dd   0   ;//  
   Reserved        dq   0   ;// <-------------- ,    64 !!!
ends

if 0

lkd> dt _BCRYPT_AUTHENTICATED_CIPHER_MODE_INFO
uxtheme!_BCRYPT_AUTHENTICATED_CIPHER_MODE_INFO
   +0x000 cbSize           : Uint4B
   +0x004 dwInfoVersion    : Uint4B
   +0x008 pbNonce          : Ptr32 UChar
   +0x00c cbNonce          : Uint4B
   +0x010 pbAuthData       : Ptr32 UChar
   +0x014 cbAuthData       : Uint4B
   +0x018 pbTag            : Ptr32 UChar
   +0x01c cbTag            : Uint4B
   +0x020 pbMacContext     : Ptr32 UChar
   +0x024 cbMacContext     : Uint4B
   +0x028 cbAAD            : Uint4B
   +0x030 cbData           : Uint8B
   +0x038 dwFlags          : Uint4B

end if

;// https://ru.bmstu.wiki/AES_(Advanced_Encryption_Standard) 

;// https://ru.wikipedia.org/wiki/Galois/Counter_Mode 

;// https://docs.microsoft.com/en-us/windows/win32/api/bcrypt/

;// https://download-dll.ru/skachat-bcrypt-dll.html












